mod connect;
mod error;
mod readers;
mod status;
mod text;
mod writers;

use std::{
    collections::HashMap,
    fmt::Debug,
    io::{Read, Write},
    net::TcpStream,
};

use error::Result;
use readers::read_string;
use status::ServerType;
use text::Message;
use varint_rs::{VarintReader, VarintWriter};

pub trait ServerState {}

#[derive(Debug)]
pub struct Disconnected;
impl ServerState for Disconnected {}
#[derive(Debug)]
pub struct Status {
    typ: ServerType,
    description: String,
    mods: Option<HashMap<String, String>>,
}
impl ServerState for Status {}
#[derive(Debug)]
#[allow(unused)]
pub struct Connected {
    typ: ServerType,
    description: String,
    uuid: String,
    username: String,
}
impl ServerState for Connected {}

#[derive(Debug)]
pub struct Server<S: ServerState + Debug> {
    stream: TcpStream,
    addr: String,
    port: u16,
    state: S,
}

impl Server<Disconnected> {
    fn new(addr: String, port: u16) -> Result<Self> {
        Ok(Self {
            stream: TcpStream::connect(format!("{addr}:{port}"))?,
            addr,
            port,
            state: Disconnected,
        })
    }
}

fn main() -> Result<()> {
    let server = Server::new("localhost".to_string(), 25565)?;
    dbg!(&server);

    let server = server.status()?;
    dbg!(&server);

    let server = server.connect()?;
    dbg!(&server);

    Ok(())
}

fn handle_keepalive(stream: &mut TcpStream, mut packet: &[u8]) {
    let _packetid = packet.read_usize_varint().unwrap();
    let mut id = [0u8; 4];
    packet.read_exact(&mut id).unwrap();

    let mut buf: Vec<u8> = Vec::new();
    buf.write_u8_varint(0).unwrap();
    buf.write_all(&id).unwrap();

    stream.write_usize_varint(buf.len()).unwrap();
    stream.write_all(&buf).unwrap();
}

fn handle_chat(mut packet: &[u8]) {
    let _packetid = packet.read_usize_varint().unwrap();

    let msg = read_string(&mut packet);
    let msg = Message::from_string(msg.unwrap());
    dbg!(msg);
}
