use serde_json::Value;

#[derive(Debug)]
pub struct Message {
    content: MessageContent,
}

#[derive(Debug)]
pub enum MessageContent {
    Text(String),
    Translate {
        translate: String,
        fallback: Option<String>,
        with: Option<Vec<Value>>,
    },
    Score {
        name: String,
        objective: String,
    },
    Selector {
        selector: String,
        seperator: Value,
    },
    Keybind(String),
    Nbt {
        source: String,
        nbt: String,
        interpret: bool,
        seperator: String,
    },
}

impl Message {
    pub fn from_string(string: String) -> Option<Self> {
        let json: Value = serde_json::from_str(&string).ok()?;
        Self::from_json(json)
    }

    fn from_json(message: Value) -> Option<Self> {
        let message = message.as_object()?;

        match message.get("type") {
            Some(typ) => match typ.as_str()? {
                "text" => Some(Message {
                    content: MessageContent::Text(message.get("text")?.as_str()?.to_string()),
                }),
                "translatable" => Some(Message {
                    content: MessageContent::Translate {
                        translate: message.get("translate")?.as_str()?.to_owned(),
                        fallback: message
                            .get("fallback")
                            .map(|v| v.as_str().map(str::to_string))
                            .flatten(),
                        with: message.get("with").map(|a| a.as_array().cloned()).flatten(),
                    },
                }),
                "score" => {
                    let score = message.get("score")?.as_object()?;
                    Some(Message {
                        content: MessageContent::Score {
                            name: score.get("name")?.as_str()?.to_string(),
                            objective: score.get("objective")?.as_str()?.to_string(),
                        },
                    })
                }
                "selector" => Some(Message {
                    content: MessageContent::Selector {
                        selector: message.get("selector")?.as_str()?.to_string(),
                        seperator: message.get("seperator")?.clone(),
                    },
                }),
                "keybind" => Some(Message {
                    content: MessageContent::Keybind(message.get("keybind")?.as_str()?.to_string()),
                }),
                "nbt" => Some(Message {
                    content: MessageContent::Nbt {
                        source: message
                            .get(message.get("source")?.as_str()?)?
                            .as_str()?
                            .to_string(),
                        nbt: message.get("nbt")?.as_str()?.to_string(),
                        interpret: message
                            .get("interpret")
                            .map(|v| v.as_bool())
                            .flatten()
                            .unwrap_or(false),
                        seperator: message
                            .get("seperator")
                            .map(|v| v.as_str())
                            .flatten()
                            .unwrap_or(", ")
                            .to_string(),
                    },
                }),
                _ => return None,
            },
            None => {
                if let Some(txt) = message.get("text") {
                    Some(Message {
                        content: MessageContent::Text(txt.as_str()?.to_string()),
                    })
                } else if let Some(translate) = message.get("translate") {
                    Some(Message {
                        content: MessageContent::Translate {
                            translate: translate.as_str()?.to_owned(),
                            fallback: message
                                .get("fallback")
                                .map(|v| v.as_str().map(str::to_string))
                                .flatten(),
                            with: message.get("with").map(|a| a.as_array().cloned()).flatten(),
                        },
                    })
                } else if let Some(score) = message.get("score") {
                    let score = score.as_object()?;
                    Some(Message {
                        content: MessageContent::Score {
                            name: score.get("name")?.as_str()?.to_string(),
                            objective: score.get("objective")?.as_str()?.to_string(),
                        },
                    })
                } else if let Some(selector) = message.get("selector") {
                    Some(Message {
                        content: MessageContent::Selector {
                            selector: selector.as_str()?.to_string(),
                            seperator: message.get("seperator")?.clone(),
                        },
                    })
                } else if let Some(keybind) = message.get("keybind") {
                    Some(Message {
                        content: MessageContent::Keybind(keybind.as_str()?.to_string()),
                    })
                } else if let Some(nbt) = message.get("nbt") {
                    Some(Message {
                        content: MessageContent::Nbt {
                            source: message
                                .get(message.get("source")?.as_str()?)?
                                .as_str()?
                                .to_string(),
                            nbt: nbt.as_str()?.to_string(),
                            interpret: message
                                .get("interpret")
                                .map(|v| v.as_bool())
                                .flatten()
                                .unwrap_or(false),
                            seperator: message
                                .get("seperator")
                                .map(|v| v.as_str())
                                .flatten()
                                .unwrap_or(", ")
                                .to_string(),
                        },
                    })
                } else {
                    None
                }
            }
        }
    }
}
